@@ -18,20 +18,33 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- * $Id: filelist_menu.h,v 1.5 2006/01/17 13:55:53 mschwerin Exp $
+ * $Id: filelist_menu.h 1238 2006-10-05 13:14:03Z mschwerin $
  *
  */
 
 #ifndef HAVE_FILELIST_MENU_H
 #define HAVE_FILELIST_MENU_H
+#ifdef __cplusplus
+extern "C" {
+#endif
 
 #include "oxine.h"
 
 int filelist_menu_is_current_menu (oxine_t * oxine);
-void filelist_menu_level_up (oxine_t * oxine);
 
-void show_video_menu_cb (void *oxine_p);
 void show_music_menu_cb (void *oxine_p);
-void show_image_menu_cb (void *oxine_p);
+void show_video_menu_cb (void *oxine_p);
+void show_media_menu_cb (void *oxine_p);
 
+void show_filelist_menu_cb (void *oxine_p);
+
+void enter_music_menu_cb (void *oxine_p);
+void enter_video_menu_cb (void *oxine_p);
+void enter_media_menu_cb (void *oxine_p);
+
+void filelist_menu_free (void);
+
+#ifdef __cplusplus
+}
 #endif
+#endif /* HAVE_FILELIST_MENU_H */
