@@ -18,13 +18,19 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- * $Id: help_menu.h,v 1.1 2005/12/08 09:34:09 mschwerin Exp $
+ * $Id: help_menu.h 1182 2006-09-22 07:53:33Z mschwerin $
  *
  */
 
 #ifndef HAVE_HELP_MENU_H
 #define HAVE_HELP_MENU_H
+#ifdef __cplusplus
+extern "C" {
+#endif
 
 void show_help_menu_cb (void *oxine_p);
 
+#ifdef __cplusplus
+}
 #endif
+#endif /* HAVE_HELP_MENU_H */
