@@ -18,12 +18,15 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: lang.h,v 1.1 2005/09/18 11:40:41 mschwerin Exp $
+ * $Id: lang.h 1379 2006-10-16 13:27:04Z mschwerin $
  *
  */
 
-#ifndef LANG_H
-#define LANG_H
+#ifndef HAVE_LANG_H
+#define HAVE_LANG_H
+#ifdef __cplusplus
+extern "C" {
+#endif
 
 /* This is just an alpha sort order. */
 typedef enum {
@@ -35,7 +38,8 @@ typedef enum {
     SPANISH,
     POLISH,
     UKRAINIAN,
-    CZECH
+    CZECH,
+    NORWEGIAN_BOKMAL
 } lang_code_t;
 
 typedef struct {
@@ -50,4 +54,7 @@ const langs_t *get_default_lang (void);
 
 const char *get_language_from_iso639_1 (char *two_letters);
 
+#ifdef __cplusplus
+}
 #endif
+#endif /* HAVE_LANG_H */
