@@ -18,25 +18,39 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- * $Id: main_menu.h,v 1.6 2006/01/08 15:40:03 mschwerin Exp $
+ * $Id: main_menu.h 1522 2006-11-11 13:57:10Z mschwerin $
  *
  */
 
 #ifndef HAVE_MAIN_MENU_H
 #define HAVE_MAIN_MENU_H
+#ifdef __cplusplus
+extern "C" {
+#endif
 
 #include "oxine.h"
 
-void shutdown_cb (void *oxine_p);
 void eject_cb (void *oxine_p);
-void show_background (oxine_t * oxine, const char *mrl);
-void show_clock (oxine_t * oxine);
-void show_main_menu_cb (void *oxine_p);
-#ifdef HAVE_DVB
-void play_dvb_cb (void *oxine_p);
-#endif
-#ifdef HAVE_V4L
-void play_v4l_cb (void *oxine_p);
-#endif
+void execute_cb (void *cmd_p);
+void shutdown_cb (void *oxine_p);
 
+void play_dvd_cb (void *oxine_p);
+void play_dvb_cb (void *oxine_p);
+void play_v4l_cb (void *oxine_p);
+#ifdef HAVE_VDR
+void play_vdr_cb (void *oxine_p);
 #endif
+void play_mrl_cb (void *mrl_p);
+void play_audiocd_cb (void *oxine_p);
+void play_vcd_cb (void *oxine_p);
+
+void show_sub_menu_cb (void *menu_mrl_p);
+void show_main_menu_cb (void *oxine_p);
+
+void main_menu_init (void);
+void main_menu_free (void);
+
+#ifdef __cplusplus
+}
+#endif
+#endif /* HAVE_MAIN_MENU_H */
