@@ -19,13 +19,21 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- * $Id: mediamarks.h,v 1.4 2005/11/17 08:46:27 mschwerin Exp $
+ * $Id: mediamarks.h 1372 2006-10-16 10:39:45Z mschwerin $
  *
  */
 
 #ifndef HAVE_MEDIAMARKS_H
 #define HAVE_MEDIAMARKS_H
-
-int mediamarks_read (filelist_t * filelist);
-
+#ifdef __cplusplus
+extern "C" {
 #endif
+
+char *read_entire_file (const char *mrl, int *file_size);
+int mediamarks_read (filelist_t * filelist);
+int favorites_read (filelist_t * filelist);
+
+#ifdef __cplusplus
+}
+#endif
+#endif /* HAVE_MEDIAMARKS_H */
