@@ -18,12 +18,15 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- * $Id: meta_info.h,v 1.2 2005/09/22 07:10:24 mschwerin Exp $
+ * $Id: meta_info.h 1465 2006-11-06 07:24:04Z mschwerin $
  *
  */
 
-#ifndef META_INFO_H
-#define META_INFO_H
+#ifndef HAVE_META_INFO_H
+#define HAVE_META_INFO_H
+#ifdef __cplusplus
+extern "C" {
+#endif
 
 typedef enum {
     META_INFO_TITLE = XINE_META_INFO_TITLE,
@@ -31,10 +34,14 @@ typedef enum {
     META_INFO_GENRE = XINE_META_INFO_GENRE,
     META_INFO_ALBUM = XINE_META_INFO_ALBUM,
     META_INFO_YEAR = XINE_META_INFO_YEAR,
-    META_INFO_TRACK_NUMBER = XINE_META_INFO_TRACK_NUMBER
+    META_INFO_TRACK_NUMBER = XINE_META_INFO_TRACK_NUMBER,
+    META_INFO_CDINDEX_DISCID = XINE_META_INFO_CDINDEX_DISCID,
 } meta_info_t;
 
 char *meta_info_get_from_stream (xine_stream_t * stream, meta_info_t type);
 char *meta_info_get_from_mrl (const char *mrl, meta_info_t type);
 
+#ifdef __cplusplus
+}
 #endif
+#endif /* HAVE_META_INFO_H */
