@@ -18,7 +18,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- * $Id: playlist.c,v 1.34 2006/01/19 09:31:32 mschwerin Exp $
+ * $Id: playlist.c 1413 2006-10-23 07:16:27Z mschwerin $
  *
  */
 #include "config.h"
@@ -32,15 +32,19 @@
 #include <sys/stat.h>
 #include <sys/types.h>
 
+#include "environment.h"
 #include "filelist.h"
 #include "heap.h"
 #include "i18n.h"
 #include "logger.h"
 #include "oxine.h"
 #include "playlist.h"
+#include "playlist_m3u.h"
+#include "playlist_xml.h"
 
 extern oxine_t *oxine;
 
+
 /* 
  * ***************************************************************************
  * Name:            playitem_free_cb
@@ -77,13 +81,20 @@ playitem_free_cb (void *data)
  * ***************************************************************************
  */
 playlist_t *
-playlist_new (void)
+playlist_new (const char *mrl, playlist_change_cb_t cb)
 {
     playlist_t *playlist = ho_new (playlist_t);
 
     playlist->list = l_list_new ();
     playlist->current = NULL;
     playlist->playmode = PLAYLIST_MODE_NORMAL;
+    playlist->top_position = 0;
+    playlist->cur_position = 0;
+    playlist->change_cb = cb;
+
+    playlist->mrl = NULL;
+    if (mrl)
+        playlist->mrl = ho_strdup (mrl);
 
     return playlist;
 }
@@ -104,6 +115,12 @@ playlist_clear (playlist_t * playlist)
 
     l_list_clear (playlist->list, playitem_free_cb);
     playlist->current = NULL;
+    playlist->playmode = PLAYLIST_MODE_NORMAL;
+    playlist->top_position = 0;
+    playlist->cur_position = 0;
+
+    if (playlist->change_cb)
+        playlist->change_cb (playlist);
 }
 
 
@@ -121,6 +138,7 @@ playlist_free (playlist_t * playlist)
     assert (playlist);
 
     l_list_free (playlist->list, playitem_free_cb);
+    ho_free (playlist->mrl);
     ho_free (playlist);
 }
 
@@ -135,8 +153,7 @@ guess_subtitle_mrl (const char *mrl)
         _mrl += 5;
 
     char autosub[1024];
-    memset (&autosub, 0, sizeof (autosub));
-    snprintf (autosub, sizeof (autosub), "%s", _mrl);
+    strcpy (autosub, _mrl);
 
     char *ending;
     if ((ending = strrchr (autosub, '.')))
@@ -160,13 +177,13 @@ guess_subtitle_mrl (const char *mrl)
             sub = ho_strdup (autosub);
 
 #ifdef DEBUG
-            char *__mrl = ho_strdup (mrl);
-            char *__sub = ho_strdup (sub);
+            char *mrl_bn = get_basename (mrl);
+            char *sub_bn = get_basename (sub);
             debug ("found subtitle file");
-            debug ("         mrl: %s", basename (__mrl));
-            debug ("    subtitle: %s", basename (__sub));
-            ho_free (__mrl);
-            ho_free (__sub);
+            debug ("         mrl: %s", mrl_bn);
+            debug ("    subtitle: %s", sub_bn);
+            ho_free (mrl_bn);
+            ho_free (sub_bn);
 #endif
 
             break;
@@ -201,17 +218,16 @@ playlist_insert (playlist_t * playlist, int position,
     if (mrl)
         playitem->mrl = ho_strdup (mrl);
 
-    int subtitle_autoload =
-        xine_config_register_bool (oxine->xine, "gui.subtitle_autoload", 1,
-                                   _("Subtitle autoloading"),
-                                   _("Automatically load subtitles "
-                                     "if they exist."), 10, NULL, NULL);
-
-    if (subtitle_autoload)
+    xine_cfg_entry_t centry;
+    xine_config_lookup_entry (oxine->xine, "subtitles.autoload", &centry);
+    if (centry.num_value)
         playitem->subtitle_mrl = guess_subtitle_mrl (mrl);
 
     l_list_insert (playlist->list, position, playitem);
 
+    if (playlist->change_cb)
+        playlist->change_cb (playlist);
+
     return playitem;
 }
 
@@ -236,17 +252,16 @@ playlist_add (playlist_t * playlist, const char *title, const char *mrl)
     if (mrl)
         playitem->mrl = ho_strdup (mrl);
 
-    int subtitle_autoload =
-        xine_config_register_bool (oxine->xine, "gui.subtitle_autoload", 1,
-                                   _("Subtitle autoloading"),
-                                   _("Automatically load subtitles "
-                                     "if they exist."), 10, NULL, NULL);
-
-    if (subtitle_autoload)
+    xine_cfg_entry_t centry;
+    xine_config_lookup_entry (oxine->xine, "subtitles.autoload", &centry);
+    if (centry.num_value)
         playitem->subtitle_mrl = guess_subtitle_mrl (mrl);
 
     l_list_append (playlist->list, playitem);
 
+    if (playlist->change_cb)
+        playlist->change_cb (playlist);
+
     return playitem;
 }
 
@@ -268,31 +283,33 @@ playlist_add_fileitem (playlist_t * playlist, fileitem_t * fileitem)
     playitem_t *playitem = NULL;
 
     switch (fileitem->type) {
-        case FILE_TYPE_REGULAR:
-            playitem =
-                playlist_add (playlist, fileitem->title, fileitem->mrl);
-            break;
-        case FILE_TYPE_DIRECTORY:
-        case FILE_TYPE_MOUNTPOINT:
-        case FILE_TYPE_MEDIAMARKS:
-        case FILE_TYPE_M3U:
-        case FILE_TYPE_AUTODISC:
-            if (fileitem->sublist) {
-                fileitem_t *subitem = filelist_first (fileitem->sublist);
-                while (subitem) {
-                    filelist_expand (fileitem->sublist, subitem);
-                    playlist_add_fileitem (playlist, subitem);
-                    subitem = filelist_next (fileitem->sublist, subitem);
-                }
-            } else {
-                fatal ("You should call have filelist_expand!");
-                abort ();
+    case FILE_TYPE_REGULAR:
+        playitem = playlist_add (playlist, fileitem->title, fileitem->mrl);
+        break;
+    case FILE_TYPE_DIRECTORY:
+    case FILE_TYPE_MOUNTPOINT:
+    case FILE_TYPE_MEDIAMARKS:
+    case FILE_TYPE_M3U:
+    case FILE_TYPE_AUTODISC:
+        if (fileitem->sublist) {
+            fileitem_t *subitem = filelist_first (fileitem->sublist);
+            while (subitem) {
+                filelist_expand (subitem);
+                playlist_add_fileitem (playlist, subitem);
+                subitem = filelist_next (fileitem->sublist, subitem);
             }
-            break;
-        default:
-            break;
+        } else {
+            fatal ("You should have called filelist_expand!");
+            abort ();
+        }
+        break;
+    default:
+        break;
     }
 
+    if (playlist->change_cb)
+        playlist->change_cb (playlist);
+
     return playitem;
 }
 
@@ -312,6 +329,9 @@ playlist_move_up (playlist_t * playlist, playitem_t * playitem)
     assert (playitem);
 
     l_list_move_up (playlist->list, playitem);
+
+    if (playlist->change_cb)
+        playlist->change_cb (playlist);
 }
 
 
@@ -330,6 +350,9 @@ playlist_move_down (playlist_t * playlist, playitem_t * playitem)
     assert (playitem);
 
     l_list_move_down (playlist->list, playitem);
+
+    if (playlist->change_cb)
+        playlist->change_cb (playlist);
 }
 
 
@@ -356,6 +379,9 @@ playlist_remove (playlist_t * playlist, playitem_t * playitem)
         ho_free (playitem->mrl);
     }
     ho_free (playitem);
+
+    if (playlist->change_cb)
+        playlist->change_cb (playlist);
 }
 
 
@@ -372,7 +398,7 @@ playlist_random_next (playlist_t * playlist)
 {
     assert (playlist);
 
-    if (!playlist->list->length)
+    if (!l_list_length (playlist->list))
         return NULL;
 
     int num_played = 0;
@@ -382,13 +408,13 @@ playlist_random_next (playlist_t * playlist)
             num_played++;
         item = l_list_next (playlist->list, item);
     }
-    if (num_played == playlist->list->length)
+    if (num_played == l_list_length (playlist->list))
         return NULL;
 
-    int found = FALSE;
+    int found = 0;
     while (!found) {
         int next =
-            (int) ((double) playlist->list->length *
+            (int) ((double) l_list_length (playlist->list) *
                    ((double) random () / (double) RAND_MAX));
         playitem_t *item = l_list_first (playlist->list);
         while (item && next > 0) {
@@ -421,23 +447,28 @@ playlist_get_first (playlist_t * playlist)
     assert (playlist);
 
     switch (playlist->playmode) {
-        case PLAYLIST_MODE_NORMAL:
-        case PLAYLIST_MODE_REPEAT:
-            playlist->current = l_list_first (playlist->list);
-            break;
-        case PLAYLIST_MODE_RANDOM:
-            srandom ((unsigned int) time (NULL));
-            playlist->current = l_list_first (playlist->list);
-            while (playlist->current) {
-                playlist->current->played = FALSE;
-                playlist->current =
-                    l_list_next (playlist->list, playlist->current);
-            }
-            playlist->current = playlist_random_next (playlist);
-            break;
+    case PLAYLIST_MODE_NORMAL:
+    case PLAYLIST_MODE_REPEAT:
+        playlist->current = l_list_first (playlist->list);
+        break;
+    case PLAYLIST_MODE_RANDOM:
+        srandom ((unsigned int) time (NULL));
+        playlist->current = l_list_first (playlist->list);
+        while (playlist->current) {
+            playlist->current->played = 0;
+            playlist->current =
+                l_list_next (playlist->list, playlist->current);
+        }
+        playlist->current = playlist_random_next (playlist);
+        break;
     }
+
     if (playlist->current)
-        playlist->current->played = TRUE;
+        playlist->current->played = 1;
+
+    if (playlist->change_cb)
+        playlist->change_cb (playlist);
+
     return playlist->current;
 }
 
@@ -455,17 +486,24 @@ playlist_get_prev (playlist_t * playlist)
 {
     assert (playlist);
 
+    if (!playlist->current)
+        return NULL;
+
     switch (playlist->playmode) {
-        case PLAYLIST_MODE_NORMAL:
-        case PLAYLIST_MODE_REPEAT:
-            playlist->current =
-                l_list_prev (playlist->list, playlist->current);
-            break;
-        default:
-            break;
+    case PLAYLIST_MODE_NORMAL:
+    case PLAYLIST_MODE_REPEAT:
+        playlist->current = l_list_prev (playlist->list, playlist->current);
+        break;
+    default:
+        break;
     }
+
     if (playlist->current)
-        playlist->current->played = TRUE;
+        playlist->current->played = 1;
+
+    if (playlist->change_cb)
+        playlist->change_cb (playlist);
+
     return playlist->current;
 }
 
@@ -484,23 +522,29 @@ playlist_get_next (playlist_t * playlist)
 {
     assert (playlist);
 
+    if (!playlist->current)
+        return NULL;
+
     switch (playlist->playmode) {
-        case PLAYLIST_MODE_NORMAL:
-            playlist->current =
-                l_list_next (playlist->list, playlist->current);
-            break;
-        case PLAYLIST_MODE_REPEAT:
-            playlist->current =
-                l_list_next (playlist->list, playlist->current);
-            if (!playlist->current)
-                playlist->current = l_list_first (playlist->list);
-            break;
-        case PLAYLIST_MODE_RANDOM:
-            playlist->current = playlist_random_next (playlist);
-            break;
+    case PLAYLIST_MODE_NORMAL:
+        playlist->current = l_list_next (playlist->list, playlist->current);
+        break;
+    case PLAYLIST_MODE_REPEAT:
+        playlist->current = l_list_next (playlist->list, playlist->current);
+        if (!playlist->current)
+            playlist->current = l_list_first (playlist->list);
+        break;
+    case PLAYLIST_MODE_RANDOM:
+        playlist->current = playlist_random_next (playlist);
+        break;
     }
+
     if (playlist->current)
-        playlist->current->played = TRUE;
+        playlist->current->played = 1;
+
+    if (playlist->change_cb)
+        playlist->change_cb (playlist);
+
     return playlist->current;
 }
 
@@ -519,8 +563,29 @@ playlist_set_current (playlist_t * playlist, playitem_t * current)
     assert (playlist);
 
     playlist->current = current;
+
     if (playlist->current)
-        playlist->current->played = TRUE;
+        playlist->current->played = 1;
+
+    if (playlist->change_cb)
+        playlist->change_cb (playlist);
+
+    return playlist->current;
+}
+
+
+/* 
+ * ***************************************************************************
+ * Name:            playlist_get_current
+ * Access:          public
+ *
+ * Description:     Returns the current pointer.
+ * ***************************************************************************
+ */
+playitem_t *
+playlist_get_current (playlist_t * playlist)
+{
+    assert (playlist);
 
     return playlist->current;
 }
@@ -538,34 +603,34 @@ playitem_t *
 playlist_set_current_pos (playlist_t * playlist, int pos)
 {
     assert (playlist);
-    assert (pos < playlist_length (playlist));
 
-    int i = 0;
-    playitem_t *cur = l_list_first (playlist->list);
-    while (cur) {
-        if (i == pos) {
-            playlist->current = cur;
-            playlist->current->played = TRUE;
-            break;
+    playitem_t *cur = NULL;
+    if ((pos >= 0) && (pos < playlist_length (playlist))) {
+        int i = 0;
+        cur = l_list_first (playlist->list);
+        while (cur) {
+            if (i == pos) {
+                break;
+            }
+            cur = l_list_next (playlist->list, cur);
+            i++;
         }
-        cur = l_list_next (playlist->list, cur);
-        i++;
     }
 
-    return playlist->current;
+    return playlist_set_current (playlist, cur);
 }
 
 
 /* 
  * ***************************************************************************
- * Name:            playlist_current_pos
+ * Name:            playlist_get_current_pos
  * Access:          public
  *
  * Description:     Returns the position of the currently playing entry.
  * ***************************************************************************
  */
 int
-playlist_current_pos (playlist_t * playlist)
+playlist_get_current_pos (playlist_t * playlist)
 {
     assert (playlist);
 
@@ -577,6 +642,7 @@ playlist_current_pos (playlist_t * playlist)
         cur = l_list_next (playlist->list, cur);
         i++;
     }
+
     return -1;
 }
 
@@ -631,3 +697,90 @@ playlist_length (playlist_t * playlist)
 
     return l_list_length (playlist->list);
 }
+
+
+/* 
+ * ***************************************************************************
+ * Name:            playlist_get_playmode
+ * Access:          public
+ *
+ * Description:     Returns the current playmode of this playlist.
+ * ***************************************************************************
+ */
+playlist_mode_t
+playlist_get_playmode (playlist_t * playlist)
+{
+    assert (playlist);
+
+    return playlist->playmode;
+}
+
+
+/* 
+ * ***************************************************************************
+ * Name:            playlist_set_playmode
+ * Access:          public
+ *
+ * Description:     Sets the current playmode of this playlist.
+ * ***************************************************************************
+ */
+void
+playlist_set_playmode (playlist_t * playlist, playlist_mode_t playmode)
+{
+    assert (playlist);
+
+    playlist->playmode = playmode;
+
+    if (playlist->change_cb)
+        playlist->change_cb (playlist);
+}
+
+
+/* 
+ * ***************************************************************************
+ * Name:            playlist_load
+ * Access:          public
+ *
+ * Description:     Load the playlist.
+ * ***************************************************************************
+ */
+bool
+playlist_load (playlist_t * playlist)
+{
+    assert (playlist);
+
+    bool res = false;
+    char *mrl = playlist->mrl;
+    if (strncasecmp (mrl + strlen (mrl) - 3, "xml", 3) == 0) {
+        res = playlist_xml_load (playlist, mrl);
+    } else {
+        res = playlist_m3u_load (playlist, mrl);
+    }
+
+    return res;
+}
+
+
+/* 
+ * ***************************************************************************
+ * Name:            playlist_save
+ * Access:          public
+ *
+ * Description:     Save the playlist.
+ * ***************************************************************************
+ */
+bool
+playlist_save (playlist_t * playlist)
+{
+    assert (playlist);
+
+    bool res = false;
+    char *mrl = playlist->mrl;
+    if (strncasecmp (mrl + strlen (mrl) - 3, "xml", 3) == 0) {
+        res = playlist_xml_save (playlist, mrl);
+    } else {
+        res = playlist_m3u_save (playlist, mrl);
+    }
+
+    return res;
+}
