@@ -19,12 +19,15 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- * $Id: scheduler.h,v 1.2 2005/09/02 07:41:52 mschwerin Exp $
+ * $Id: scheduler.h 1182 2006-09-22 07:53:33Z mschwerin $
  *
  */
 
 #ifndef HAVE_SCHEDULER_H
 #define HAVE_SCHEDULER_H
+#ifdef __cplusplus
+extern "C" {
+#endif
 
 void start_scheduler (void);
 void stop_scheduler (void);
@@ -35,4 +38,7 @@ void cancel_job (int job_id);
 void lock_job_mutex (void);
 void unlock_job_mutex (void);
 
+#ifdef __cplusplus
+}
 #endif
+#endif /* HAVE_SCHEDULER_H */
